import java.lang.*;
import java.util.*;
import java.io.*;

class PromotionResolver {

	/**
	 *  Vector of Strings which are the same as those currently selected
	 */
	private Vector activePromotions_;
	public Vector activePromotions() { return activePromotions_; }
	public void activePromotions( Vector aVector ) { activePromotions_ = aVector; }

	/**
	 *  Promotions for this store .. includes ones that are not active
	 */
	private Hashtable promotions_;
	public Hashtable promotions() { return promotions_; }
	public void promotions( Hashtable anHT ) { promotions_ = anHT; }

    PromotionResolver() {
        this.initialize();
    }

    private void initialize() {
        this.promotions(new Hashtable());
        this.activePromotions(new Vector());
        this.setupPromotions();
    }

    public void setupPromotions() {
        
        this.addPromotionsFrom(this.promotionClassFileNames());
/*
        // PromoTuesday
        PromoTuesday p1 = new PromoTuesday();
        this.promotions().put(p1.name(), p1);

        // PromoKids
        PromoKids p2 = new PromoKids();
        this.promotions().put(p2.name(), p2);

        // PromoFive
        PromoFive p3 = new PromoFive();
        this.promotions().put(p3.name(), p3);
*/
    }

    public Enumeration promotionNames() {
        return this.promotions().keys();
    }


    public double computeBestPrice( Customer aCustomer ) {
        double price = aCustomer.basket().regularCost();
        System.out.println("price before promos = " + price);
        Enumeration enum = this.activePromotions().elements();
        while( enum.hasMoreElements() ){
            double aPrice = ((Promotion) enum.nextElement()).
                                        promoPrice(aCustomer);
            if( aPrice < price ) price = aPrice;
        }

        return price;
    }

    public void setActivePromotions( Vector promoNames ){
        Vector promos = new Vector();
        Enumeration enum = promoNames.elements();
        while( enum.hasMoreElements() ){
            String aPromoName = (String) enum.nextElement();
            promos.addElement(this.promotions().get(aPromoName));
        }
        this.activePromotions( promos );
    }
    
    public Vector promotionClassFileNames() {
        Vector classFileNames = new Vector();
        File theFile = new File(".");
        String dir[] = theFile.list();
        for (int i=0; i< dir.length; i++) {
            String theEntry = dir[i];
            if ( (theEntry.startsWith("Promo")) 
                     && (!theEntry.startsWith("Promotion"))
                         && (theEntry.endsWith(".class"))) {
                System.out.println("promotion class = " + theEntry);
                StringTokenizer s = new StringTokenizer(theEntry, ".");
                classFileNames.addElement(s.nextToken());
            }
        }
        return classFileNames;
    }
    
    public void addPromotionsFrom(Vector thePromotionClassFileNames) {
        
        Promotion thePromotion = null;
        Enumeration theOC = thePromotionClassFileNames.elements();
        
        while (theOC.hasMoreElements()) {
            String theClassName = (String) theOC.nextElement();
            try {
                thePromotion = ((Promotion) ((Class.forName(theClassName)).newInstance()));
            }
            catch (InstantiationException e)
                {System.out.println("Promotion Creation Error InstantiationException"); }
            catch (ClassNotFoundException e)
                {System.out.println("Promotion Creation Error ClassNotFoundException"); }
            catch (IllegalAccessException e)
                {System.out.println("Promotion Creation Error IllegalAccessException"); }
                
            this.promotions().put(thePromotion.name(), thePromotion);
        }
            
    }
        
}